function K = beamutil( flag, E, I, L)

% SYNOPSIS:
%
% K = beamutil( flag, E, I, L)
%
% Builds a two-dimensional beam stiffness matrix (4x4).
% In the future, this will do other things as well...
%
% flag:
%
%   'linear'           - linear stiffness matrix, most common case
%   'plastic-elastic'  - stiffness matrix when yielding is occuring at node 1
%   'elastic-plastic'  - stiffness matrix when yielding is occuring at node 2
%
% For three dimensional problems a 'y' or 'z' should be appended to the
%	flags above if the moment of inertia is about the y- or z-axis,
% respectively.  The coordinate systems are slightly different for the
% two cases.
%
% Note that Iz is used when determing the forces in the y-directions and
% moments about the z-axis.  Similarly, Iy is used for the forces in the
% z-direction and moments about the y-axis.

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

% {TEX} Utility function for beam elements

% Transformation matrix needed for bending in x-z plane
T = diag([1 -1 1 -1]);   % Note: inv(T) = T

switch flag
	case {'linear', 'linearz'}
		K = linear_K( E, I, L);
	case 'lineary'  % `y' refers to Theta_y, which means x-z bending
		K = T * linear_K( E, I, L) * T;
	case {'plastic-elastic', 'plastic-elasticz'}
		K = nl_Kpe( E, I, L);
	case 'plastic-elasticy'
		K = T * nl_Kpe( E, I, L) * T;
	case {'elastic-plastic', 'elastic-plasticz'}
		K = nl_Kep( E, I, L);
	case 'elastic-plasticy'
		K = T * nl_Kep( E, I, L) * T;
	otherwise
		warning(['Beam3d: Unknown option ''' flag '''.']);
end

function  a = linear_K( E, I, L)
	a = (E * I / L^3) * [  12,   6*L,   -12,    6*L ; ...
												6*L, 4*L^2,  -6*L,  2*L^2 ; ...
												-12,  -6*L,    12,   -6*L ; ...
												6*L, 2*L^2,  -6*L,  4*L^2];

function  a = nl_Kpe( E, I, L)
	a = (E * I / L^3) * [   3,     0,   - 3,    3*L ; ...
													0,     0,     0,      0 ; ...
												- 3,     0,     3,   -3*L ; ...
												3*L,     0,  -3*L,  3*L^2];

function  a = nl_Kep( E, I, L)
	a = (E * I / L^3) * [   3,   3*L,   - 3,      0 ; ...
												3*L, 3*L^2,  -3*L,      0 ; ...
												- 3,  -3*L,     3,      0 ; ...
													0,     0,     0,      0];


%  GNUFEM -- A free finite element code for Matlab
%
% Licence Details and Credits:
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
