function  [K, data] = assemble (data, flag)
%
% SYNOPSIS
%
% [K, data, C] = assemble (data, flag)
%
% Assembles the initial stiffness matrix (before boundary
% conditions are applied).
%
% Also returns a modified form of data -- the stiffness matrices
% are saved for all elements that return type.keep_K for the
% 'element-type' flag.
%
% flag is used to specify options for the assembly
%    'stiffness'  - assemble the stiffness matrix (default)
%    'damping'    - assemble a damping matrix using elements that report
%                   themselves as a 'damping_element'

if nargin < 2
	flag = 'stiffness-matrix';
end

switch flag
	case 'damping-matrix'
		[K,data] = assemble_damping( data);
	otherwise
		[K,data] = assemble_stiffness( data);
end
	

function [K,data] = assemble_stiffness( data)

K = zeros (data.num_dofs * data.num_nodes);

%  Loop the elements and build K
for i = 1 : data.num_elements

	if isfield( data.elements(i), 'K') & ~ isempty(data.elements(i).K)
		Klocal = data.elements(i).K;
	else

		type = feval( data.elements(i).type, 'element-type');

		% Use the pre_yield_stiffness matrix if desired for nonlinear elements
		if isfield(type, 'nonlinear') & type.nonlinear ...
				& isfield( data, 'pre_yield_stiffness') & data.pre_yield_stiffness
			Klocal = elget( i, data, 'pre-yield-stiffness-matrix');
		else
			Klocal = elget( i, data, 'stiffness-matrix');
		end

		T = elget( i, data, 'transformation-matrix');

		Klocal = T' * Klocal * T;

		if isfield(type, 'keep_K') & type.keep_K
			data.elements(i).K = Klocal;
		end

	end

	% Assemble local -> Global
	trans_vec = dofvec( data.elements(i), data.num_dofs, data.dofs_pos);
	K(trans_vec,trans_vec) = K(trans_vec,trans_vec) + Klocal;

end  % for i

function [C,data] = assemble_damping(data)
C = zeros (data.num_dofs * data.num_nodes);
%  Loop the elements and build C
for i = 1 : data.num_elements

	type = feval( data.elements(i).type, 'element-type');
	if isfield( type, 'damping_element') & type.damping_element

		if isfield( data.elements(i), 'C') & ~ isempty(data.elements(i).C)
			Clocal = data.elements(i).C;
		else
			Clocal = elget( i, data, 'damping-matrix');
			T = elget( i, data, 'transformation-matrix');
			Clocal = T' * Clocal * T;
		end

		% Assemble local -> Global
		trans_vec = dofvec( data.elements(i), data.num_dofs, data.dofs_pos);
		C(trans_vec,trans_vec) = C(trans_vec,trans_vec) + Clocal;

	end

end % for i
%  GNUFEM -- A free finite element code for Matlab
%
% Licence Details and Credits:
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
