function varargout = beam2d( flag, varargin)    % TEXFEM

% SYNOPSIS:
%
% function varargout = beam2d(varargin)
%
% Two-dimensional beam element for use with the FE toolkit.
%
% See element_definition for details

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

% {TEX} This is a two-node, two-dimensional, linear element.

switch flag
	case 'stiffness-matrix'
		varargout{1} = local_stiff(varargin{:});
	case 'transformation-matrix'
		varargout{1} = local_transform(varargin{:});
	case 'num-dof'
		varargout{1} = 3;   % three degrees of freedom per node
	case 'num-nodes'
		varargout{1} = 2;   % two nodes per element
	case 'dof-vector'  % equivalent to the dofs array of a FElt definition
		varargout{1} = [1 2 6];
	case {'element-forces','element-stress'}
		varargout{1} = local_stress( varargin{:});
	case {'fast-element-forces'}
		varargout{1} = fast_local_forces( varargin{:});
	case 'element-forces-fast'   % Stiffness matrix passed in instead of assembled
		varargout{1} = local_forces_fast( varargin{:});
	case 'element-type'
		type.keep_K = 1;  % note: the *transformed* matrix is stored (T' * K_loc * T)
		type.forces = 1;
		varargout{1} = type;
	otherwise
		warning(['Beam2d: Unknown option ''' flag '''.']);
end

% --------------------------------------------------------------------------

function K = local_stiff(element, material, section, nodelist)

	L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));

	if  L < eps 
		error ('length of element is zero to machine precision'); 
	end

	AEonL = section.A * material.E / L;

	% Required Material Fields
	% {TEXRM} $E$
	% Required Section Fields
	% {TEXRS} $A$, $I$

	K([2,3,5,6],[2,3,5,6]) = beamutil( 'linear', material.E, section.I, L);
	K(1,1) =     AEonL;   K(1,4)  =    -AEonL;
	K(4,1) =    -AEonL;   K(4,4) =     AEonL;

function T = local_transform(element, nodelist)

	L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));

	if  L < eps 
		error ('length of element is zero to machine precision'); 
	end

	dx = (nodelist( element.nodes(2),1:2) - nodelist( element.nodes(1),1:2)) / L;
	cx = dx(1);
	cy = dx(2);

	T = sparse( 6,6);
	T(1, 1) = cx;
	T(1, 2) = cy;
	T(2, 1) = -cy;
	T(2, 2) = cx;
	T(3, 3) = 1;
	T(4, 4) = cx;
	T(4, 5) = cy;
	T(5, 4) = -cy;
	T(5, 5) = cx;
	T(6, 6) = 1;

% compute the element forces
% D are the *global* displacements
function f = local_stress( element, material, section, nodelist, D)

Klocal = local_stiff(element, material, section, nodelist);
T = local_transform( element, nodelist);

f = Klocal * T * D;
f = matrix_clean(f);

function f = fast_local_forces( element, material, section, nodelist, D, KT)

f = KT * D;

% {TEX} \input{nlfet/beam2d}

% Licence Details and Credits:

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
