function varargout = beam3d( flag, varargin)    % TEXFEM

% SYNOPSIS:
%
% function varargout = beam3d(varargin)
%
% Three-dimensional beam element for use with the FE toolkit.
%
% See element_definition for details

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

% {TEX} This is a two-node, three-dimensional, linear element.

switch flag
	case 'stiffness-matrix'
		varargout{1} = local_stiff(varargin{:});
	case 'transformation-matrix'
		varargout{1} = local_transform(varargin{:});
	case 'num-dof'
		varargout{1} = 6;   % six degrees of freedom per node
	case 'num-nodes'
		varargout{1} = 2;   % two nodes per element
	case 'dof-vector'  % equivalent to the dofs array of a FElt definition
		varargout{1} = [1 2 3 4 5 6];
	case {'element-forces','element-stress'}
		varargout{1} = local_stress( varargin{:});
	case {'fast-element-forces'}
		varargout{1} = fast_local_forces( varargin{:});
	case 'element-forces-fast'   % Stiffness matrix passed in instead of assembled
		varargout{1} = local_forces_fast( varargin{:});
	case 'element-type'
		type.keep_K = 1;  % note: the *transformed* matrix is stored (T' * K_loc * T)
		type.forces = 1;
		varargout{1} = type;
	otherwise
		warning(['Beam3d: Unknown option ''' flag '''.']);
end

% --------------------------------------------------------------------------

function K = local_stiff(element, material, section, nodelist)

	L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));

	if  L < eps 
		error ('length of element is zero to machine precision'); 
	end

	% Required Material Fields
	% {TEXRM} $E$, $G$ or $nu$
	% Required Section Fields
	% {TEXRS} $Iy$, $Iz$, $J$, $A$

	if isfield( material, 'G') & ~ isempty( material.G)
		GJ  = material.G * section.J;
		if isfield( material, 'nu') & ~ isempty(material.nu)
			% Check that properties are consistent
			if abs( material.G - material.E/(2*(1+material.nu))) > eps
				warning( 'G, E, and nu are not consistent');
			end
		end
	elseif isfield( material, 'nu')
		GJ  = material.E * section.J / ( 2 * (1 + material.nu));  
	else
		error( 'Neither G nor nu defined for beam3d element');
	end
	AEonL = (material.E * section.A)/L;
	if abs( section.J - section.Iy - section.Iz) > eps
		warning( 'J, Iy, and Iz are not consistent');
	end

	K = zeros(12);
	yvec = [3,5,9,11];   % degrees of freedom which depend on Iy
	zvec = [2,6,8,12];   % degrees of freedom which depend on Iz
	K(yvec,yvec) = beamutil('lineary', material.E, section.Iy, L);
	K(zvec,zvec) = beamutil('linearz', material.E, section.Iz, L);
	K(1,1) =     AEonL;   K(1,7)  =    -AEonL;
	K(4,4) =      GJ/L;   K(4,10) =     -GJ/L;
	K(7,1) =    -AEonL;   K(7,7) =     AEonL;
	K(10,4) =    -GJ/L;   K(10,10) =    GJ/L;

	%K = matrix_mirror( K);

function T = local_transform(element, nodelist)

	T = zeros (12);

	L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));

	if  L < eps 
		error ('length of element is zero to machine precision'); 
	end

	dv = (nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:)) / L;
	cl = dv(1);
	cm = dv(2);
	cn = dv(3);

	d = sqrt (cl*cl + cm*cm);

	T(1,1:3) = [ cl cm cn ];
	if d <= eps
		T(2:3,1:3) = [ 0  1  0  ; ...
		               1  0  0  ];
	% Note:  I believe there is a bug in FElt for the transformation
	%        matrix.  See beam3d.c, line 339- for details.
		if cn > eps  % really, cn should be -1 or 1 so need not worry about eps
			T(3,1) = -1;
		end
	else
		T(2:3,1:3) = [ -cm/d       cl/d   0 ; ...
		               -cl*cn/d -cm*cn/d  d];
	end

	T(4:6,4:6)     = T(1:3,1:3);
	T(7:9,7:9)     = T(1:3,1:3);
	T(10:12,10:12) = T(1:3,1:3);

	T = matrix_clean(T);

	if isfield( element, 'beta') & ~isempty(element.beta)
		T = x_rotation( element.beta) * T;
		T = matrix_clean(T);
	end

% Now implement the rotation - See Balfour (1992)
function Trot = x_rotation( beta)
	Trot = zeros(12);
	Trot(1,1)     = 1;
	Trot(2:3,2:3) = [ cos( beta * pi / 180), sin( beta * pi / 180) ; ...
	                 -sin( beta * pi / 180), cos( beta * pi / 180) ];
	Trot(4:6,4:6)     = Trot(1:3,1:3);
	Trot(7:9,7:9)     = Trot(1:3,1:3);
	Trot(10:12,10:12) = Trot(1:3,1:3);
	Trot = matrix_clean(Trot);

% compute the element forces
% D are the *global* displacements
function f = local_stress( element, material, section, nodelist, D)

Klocal = local_stiff(element, material, section, nodelist);
T = local_transform( element, nodelist);

f = Klocal * T * D;
f = matrix_clean(f);

function f = fast_local_forces( element, material, section, nodelist, D, KT)

f = KT * D;

% {TEX}
% {TEX} Element axes are defined as outlined in Balfour (1992).
% {TEX} The element $x$ axis lies on the line joining the element nodes.
% {TEX} The $y$ axis is then oriented to lie in the global $x$,$y$ plane.
% {TEX} The $z$ axis is then defined using the ``right hand rule.''
% {TEX}
% {TEX} Typically, w-sections used for beams will have $I_y > I_z$.


% Licence Details and Credits:

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
