function [M3,K3] = combine_rot( M, K, nodes)
% 
% Synopsis:
% [Mcomb, Kcomb] = combine_rot( m, k, nodes)
%
% Combines mass (`m') and stiffness (`k') matrix based on the 
% geometry and story level given in `nodes'.
%
% `nodes' is a matrix defining the geometry of the nodes and
% assigning them to stories. The entries are (dx, dy, n) where dx is
% the distance in the x-direction from the center of mass of the
% floor to the node, dy is the distance in the y-direction, and n is
% the story number.  Each node must have an entry in the matrix,
% thus nodes is a Nx3 matrix where `N' is the number of stories
%
% This routine assumes that nodal rotations are constrained to match
% the rotation of the slab.
%
% `m' is assumed to be lumped mass with all mass
% concentrated on the diagonal.

numstory = max(nodes(:,3));   % number of stories

K3 = zeros( numstory*3);
M3 = zeros( numstory*3);

% Now, sum up the common floors to reduce the stiffness to size
% 3 * numstory x 3 * numstory

% loop through and fill each row of the matrices
% Note, M3 is only filled for translational mass.
for curstory = 1 : numstory
	% Loop through the nodes and pick out correct equations
	K3r = (curstory-1)   * 3 + 1;   % starting row for entries in K3
	for curnode = 1 : size( nodes,1)
		if( abs(nodes(curnode,3) - curstory) < 0.1)  % 0.1 to avoid rounding error
			% If we're here, the current node is part of the current story

			% Starting row for entries in K
			Kr  = (curnode-1)    * 3 + 1;   % row in K

			% Add the nodal mass in the x-direction and y-directions
			M3(K3r,K3r) = M3(K3r,K3r) + M(Kr,Kr);
			M3(K3r+1,K3r+1) = M3(K3r+1,K3r+1) + M(Kr+1,Kr+1);

			% loop through the nodes and add to equations for curstory
			for l = 1 : size( nodes,1)
				% sum up the x-forces to get X-Force at center of mass
				K3c = (nodes(l,3)-1) * 3 + 1;  % Columns for K3 and K
				Kc  = (l-1)          * 3 + 1;
				K3(K3r,K3c:K3c+2) = K3(K3r,K3c:K3c+2) + K(Kr,Kc:Kc+2);

				% Add in the moments due to the nodal x-forces
				% i.e.  M = f1 * - dy1 + f2 * dx1 + f3 ...
				% vpa(K(Kr,Kc:Kc+2) * nodes(l,2),5)
				K3(K3r+2,K3c:K3c+2) = K3(K3r+2,K3c:K3c+2) - K(Kr,Kc:Kc+2) * nodes(curnode,2);

				% Now, do the y-forces (f2)
				K3(K3r+1,K3c:K3c+2) = K3(K3r+1,K3c:K3c+2) + K(Kr+1,Kc:Kc+2);

				% y-force moments
				K3(K3r+2,K3c:K3c+2) = K3(K3r+2,K3c:K3c+2) + K(Kr+1,Kc:Kc+2) * nodes(curnode,1);

				% Lastly, nodal moments just add (f3)
				K3(K3r+2,K3c:K3c+2) = K3(K3r+2,K3c:K3c+2) + K(Kr+2,Kc:Kc+2);

				% That's all folks!
			end
		end
	end
end

%  GNUFEM -- A free finite element code for Matlab
%
% Licence Details and Credits:
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
