function [Meff, Keff, massvec, masslessvec, K00, K01, K10, K11] = condense( m, k)
% 
% Synopsis:
% [Meff, Keff, massvec, masslessvec, K00, K01] = condense( m, k)
%
% Condenses a structural system and returns the effective
% mass and stiffness matrices (Meff and Keff, respectively)
% after condensing out zero mass degrees of freedom.
%
% A row vector (massvec) is also returned which contains the degrees
% of freedom that were used in the condensation.
%
% `m' is assumed to be lumped mass with all mass
% concentrated on the diagonal.

% Get the indices of the non-zero diagonals of the mass matrix
nonzero_diag = find(diag(m));
% Get the indices of the zero diagonals of the mass matrix
zero_diag = find(diag(m)==0);

% make sure these are row vectors
if size(nonzero_diag, 1) < size(nonzero_diag, 2)
	nonzero_diag = nonzero_diag';
end
if size(zero_diag, 1) < size(zero_diag, 2)
	zero_diag = zero_diag';
end

% Now, build Meff and Keff
Meff = m(nonzero_diag, nonzero_diag);

% Keff is a little harder...
% The following is based roughly on the partitioning given by
% Clough and Penzien in Structural Dynamics

% Reorder the stiffness matrix so that DOFs with mass will
% go into the lower right block -- This will be known as K11

K00  = k(zero_diag,    zero_diag);
K01  = k(zero_diag,    nonzero_diag);
K10  = k(nonzero_diag, zero_diag);
K11  = k(nonzero_diag, nonzero_diag);
Keff = K11 - K10 * inv(K00) * K01;

massvec = nonzero_diag;
masslessvec = zero_diag;

%  GNUFEM -- A free finite element code for Matlab
%
% Licence Details and Credits:
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
