function femplot (data, opts, disp)
% Synopsis:
% femplot( data, opts, disp)
%
% Input:
% data   - problem data structure
% disp   - (optional) nxm matrix with nodal displacements
%          which will be plotted on top of the undeformed structure
%          n is the number of nodes, m is the number of degrees of freedom
% opts   - (optional) structure with display options
%
% Output:
% Plots the geometry of the structure
%
% Display Options:
%
% no_text           - Do not show materials, sections, etc. on the plot.
% no_center_of_mass - Do not plot center of mass
%
% To plot displacements without specifying display options:
%     femplot( data, [], disp)

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

figure(1);
hold off;

% defaults
plot_text = 1;
center_of_mass = 1;

if nargin > 1 & ~ isempty(opts)
	if isstruct(opts)
		% set up display variables based on options
		if isfield( opts, 'no_text')
			plot_text = 0;
		end
		if isfield( opts, 'no_center_of_mass')
			center_of_mass = 0;
		end
	else
		warning( 'femplot: display options (opts) must be a structure');
	end
end
for index = 1 : data.num_nodes
	node_num_string(index,1:3) = sprintf( '%3d', index);
end

if sum(data.dofs_pos(1:3) ~= 0) > 2
	plot3d = 1;
else
	plot3d = 0;
end

% Check to see if there is any variation in the z-direction
if isempty( find(abs(data.nodelist(:,3) - data.nodelist(1,3)) > eps))
	plot3d = 0;   % No depth...
end

% Plot the nodes
if plot3d
	plot3(data.nodelist(:,1),data.nodelist(:,2),data.nodelist(:,3),'b.');
else
	plot(data.nodelist(:,1),data.nodelist(:,2),'b.');
end

hold;

if plot_text
	% Set nodal text labels
	if plot3d
		text ( data.nodelist(:,1), data.nodelist(:,2),  ...
			data.nodelist(:,3), node_num_string);
	else
		text ( data.nodelist(:,1), data.nodelist(:,2), node_num_string);
	end
end


% plot the elements
for i = 1 : data.num_elements
	n = data.elements(i).nodes;
	el_name = [ num2str(i) ': '];
	if isfield( data.materials(data.elements(i).material), 'name')
		el_name = [ el_name data.materials(data.elements(i).material).name ' '];
	end
	if isfield( data.sections(data.elements(i).section), 'name')
		el_name = [ el_name data.sections(data.elements(i).section).name ];
	end
	if plot3d
		plot3( data.nodelist(n,1), data.nodelist(n,2), data.nodelist(n,3),'r');
		if nargin > 2 % plot displacements
			plot3( data.nodelist(n,1) + disp(n,1), ...
			       data.nodelist(n,2) + disp(n,2), ...
			       data.nodelist(n,3) + disp(n,3), 'm*--');
		end
		if plot_text
			% try to center the element name within the element
			h = text( sum(data.nodelist(n,1)) / length( data.nodelist(n,1)), ...
				sum(data.nodelist(n,2)) / length( data.nodelist(n,2)), ...
				sum(data.nodelist(n,3)) / length( data.nodelist(n,3)), ...
				el_name);
		end
	else
		plot( data.nodelist(n,1), data.nodelist(n,2), 'r');
		if plot_text
			h = text( sum(data.nodelist(n,1)) / length( data.nodelist(n,1)), ...
				sum(data.nodelist(n,2)) / length( data.nodelist(n,2)), ...
				el_name);
		end
	end
	if plot_text
		% set the text handle properties correctly
		set( h, 'HorizontalAlignment', 'center');
	end
end

% Plot the center of mass location
if isfield( data, 'center_of_mass') & center_of_mass
	if plot3d
		plot3( data.center_of_mass(:,1), data.center_of_mass(:,2), ...
			data.center_of_mass(:,3), 'ko');
	else
		plot3( data.center_of_mass(:,1), data.center_of_mass(:,2), 'ko');
	end
end

title('Undeformed Geometry')
axis equal

hold off

%  Plot elements


% Boundary conditions will come later
%text(0,-1,    'White "*" denotes a node.')
%text(0,-2.0, 'Red "o" denotes x-direction fixed.')
%text(0,-3.0,  'Green "+" denotes y-direction fixed.')
%text(0,-4.0, 'Blue "x" denotes moment fixed.')

%for i = 1 : data.num_nodes,
   %if data.BC(1,i)==1; plot( data.node_coords(1,i), data.node_coords(2,i),'ro'); end
   %if data.BC(2,i)==1; plot( data.node_coords(1,i), data.node_coords(2,i),'g+'); end
	 %if size( data.BC, 1) > 2
   	%if data.BC(3,i)==1;plot( data.node_coords(1,i), data.node_coords(2,i),'bx'); end
	%end
%end

% Licence Details

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
