function  [dofs_pos, dofs_num, ndof] = findDOFs (elements)
%
% SYNOPSIS
%
% [dofs_pos, dofs_num, ndof] = findDOFs (elements)
%
% elements - Array of element structures
% dofs_pos - 1x6 Vector of active degrees of freedom
% dofs_num - 1x6 vector of degree of freedom numbers
% ndof     - number of degrees of freedom, equivalent to length(find(dofs_num))
%
% Searches through all of the elements for a problem and determine which DOFs
% (out of the six that are physically possible) must be considered in this
% problem based on the different element types.
%
% Example:
%
% For a problem with only 2D beam elements (degrees of freedom are
% x-displacement, y-displacement, and z-rotation):
% dofs_pos = [ 1 2 0 0 0 3]
% dofs_num = [ 1 2 6 0 0 0]
% ndofs    = 3;

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

flag     = zeros(1,6);

last_element = '';    % keep track of the last element so we can skip
                      % if we come across an identical element.

% Go through all the elements to find active degrees of freedom
for i = 1 : length(elements)
	new_element = elements(i).type;
	if( isempty(last_element) | ~ strcmp(new_element, last_element))
		ndof = feval( new_element, 'num-dof');
		dof_vector = feval( new_element, 'dof-vector');
		for j = 1 : ndof
			flag( dof_vector(j)) = 1;
		end   % for j
		last_element = new_element;
	end % if
end % for i

dofs_pos = zeros(1,6);
dofs_num = zeros(1,6);

ndof = 0;

for i = 1 : 6
	if flag(i) > 0
		ndof = ndof + 1;
		dofs_pos(i) = ndof;
		dofs_num (ndof) = i;
	end
end

%  GNUFEM -- A free finite element code for Matlab
%
% Licence Details and Credits:
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
