function varargout = sbeam3d( flag, varargin)    % TEXFEM

% SYNOPSIS:
%
% function varargout = sbeam3d(flag, varargin)
%
% Three-dimensional simplified beam element for use with the FE toolkit.
%
% See element_definition for details

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

% {TEX} This is a two-node, three-dimensional, linear element.
% {TEX} It is a ``simplified'' element because axial torsion and
% {TEX} stiffness are neglected when forming the stiffness matrix.

switch flag
	case 'stiffness-matrix'
		varargout{1} = local_stiff(varargin{:});
	case 'transformation-matrix'
		varargout{1} = beam3d( flag, varargin{:});
	case 'num-dof'
		varargout{1} = 6;   % six degrees of freedom per node
	case 'num-nodes'
		varargout{1} = 2;   % two nodes per element
	case 'dof-vector'  % equivalent to the dofs array of a FElt definition
		varargout{1} = [1 2 3 4 5 6];
	case 'element-stress'
		varargout{1} = local_stress( varargin{:});
	case 'element-forces-fast'   % Stiffness matrix passed in instead of assembled
		varargout{1} = local_stress_fast( varargin{:});
	case 'element-type'
		varargout{1} = beam3d( flag);
	otherwise
		warning(['sbeam3d: Unknown option ''' flag '''.']);
end

% --------------------------------------------------------------------------

function K = local_stiff(element, material, section, nodelist)

	L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));
	if  L < eps 
		error ('length of element is zero to machine precision'); 
	end

	% Required Material Fields
	% {TEXRM} $E$
	% Required Section Fields
	% {TEXRS} $Iy$, $Iz$

	K = zeros(12);
	yvec = [3,5,9,11];   % degrees of freedom which depend on Iy
	zvec = [2,6,8,12];   % degrees of freedom which depend on Iz
	K(yvec,yvec) = beamutil('lineary', material.E, section.Iy, L);
	K(zvec,zvec) = beamutil('linearz', material.E, section.Iz, L);

function f = local_stress( element, material, section, nodelist, D)

Klocal = local_stiff(element, material, section, nodelist);
T = beam3d( 'transformation-matrix', element, nodelist);

f = Klocal * T * D;
f = matrix_clean(f);

function f = local_stress_fast( element, material, section, nodelist, D, KT)

f = KT * D;

% THAT'S ALL FOLKS

% {TEX}
% {TEX} This element is identical to the beam3d element except that axial
% {TEX} and torsional stiffness is neglected.
% {TEX} Note that the lack of axial stiffness may requires the use of a
% {TEX} constraint or a truss element for numerical stability. 


% Licence Details and Credits:

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
