
function varargout = unlsbeamspring3d( flag, varargin)     

% SYNOPSIS:
%
% function varargout = unlsbeam[spring]3d(varargin)
%
% Three-dimensional, uniaxial-yielding, non-linear, beam element for
% use with the GNUFEM.
%
% unlsbeamspring3d ignores any z-displacement, so that the element
% acts like a coupled pair of nonlinear springs.
%
% See element_definition for details

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

switch flag
	case 'pre-yield-stiffness-matrix'
		varargout{1} = pre_yield_stiffness_matrix( varargin{:});
	case 'stiffness-matrix'
		varargout{1} = post_yield_stiffness_matrix( varargin{:});
	case 'nonlinear-data'
		% Element defined data
		varargout{1} = nonlinear_data( varargin{:});
	case 'nonlinear-global-force'
		% Returns [f, phidot]
		%TODO: add extra fields if needed (J,G,A)
		[varargout{1},varargout{2},varargout{3}] = nonlinear_global_force( varargin{:});
	case 'transformation-matrix'
		varargout{1} = beam3d( 'transformation-matrix', varargin{:});
	case 'num-dof'
		varargout{1} = 6;   % six degrees of freedom per node
	case 'num-nodes'
		varargout{1} = 2;   % two nodes per element
	case 'num-states'
		varargout{1} = 2;   % rotations "inside" plastic hinge
	case 'dof-vector'  % equivalent to the dofs array of a FElt definition
		varargout{1} = [1 2 3 4 5 6];
	case {'element-stress', 'element-forces'}
		varargout{1} = local_forces( varargin{:});
	case 'current-state'  % varargin(1) = u, (2) = v, (3) = z, (4) = nl_data
		nld = varargin{4};
		u = nld.T * varargin{1}; v = nld.T * varargin{2}; % Transform to local
		varargout{1} = local_get_state( u, v, varargin{3}, nld.Y, nld.L, nld.My);
	case 'element-type'
		type = beam3d( flag);
		type.nonlinear = 1;
		varargout{1} = type;
	otherwise
		warning(['nlsBeam3d: Unknown option ''' flag '''.']);
end

% --------------------------------------------------------------------------

function K = post_yield_stiffness_matrix( element, material, section, nodelist)

	% Bending about the z-axis remains linear throughout
	L  = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));
	K = zeros(12);

	K([2,6,8,12],[2,6,8,12]) = beamutil('linearz', material.E, section.Iz, L);

		if isfield( material, 'alpha') & ~ isempty(material.alpha)
			K = K + material.alpha * pre_yield_stiffness_matrix( element, ...
				material, section, nodelist);
		else
			%assume perfectly plastic
			K = zeros(12);
		end

function K = pre_yield_stiffness_matrix( element, material, section, nodelist)
	K = sbeam3d( 'stiffness-matrix', element, material, section, nodelist);

	% pick off the z rows and columns and set them to zero
	K([3 9], [3 9]) = 0;

function [F, zdot, state] = nonlinear_global_force( element, material, section, nodelist, u, v, z, nl_data)

	% Transform to local
	u = nl_data.T * u;
	v = nl_data.T * v;

	state = local_get_state( u, v, z, nl_data.Y, nl_data.L, nl_data.My);

	% Put into "2D" variables to make it easier
	% These are negative because the axis for the "2D" variables is opposite
	% the pertinent axis in three-dimensional coordinates.
	theta_A_dot = - v( 5);
	theta_B_dot = - v(11);

	% change theta_y_[ab] to phi_[ab]
	u( 5) = - z(1);
	u(11) = - z(2);

	% It is possible that the moment may surpass the yield moment.  However,
	% it should not surpass by much using the ode* functions since they will
	% back up when a discontinuity in slope is detected.
	F = nl_data.Tinv * nl_data.Kee * u;
	switch state
		case 1
			zdot = [ theta_A_dot; theta_B_dot ];
		case 2

			zdot = [ theta_A_dot;  - theta_A_dot / 2];

		case 3

			zdot = [ - theta_B_dot / 2; theta_B_dot];

		case 4

			zdot = [0; 0];

		otherwise
			error('Invalid state returned from local_get_state');
	end

function state = local_get_state( u, v, z, Y, L, My)

% NOTE: u and v *must* be local coordinates.

	% u(2) = y_A  u(3) = z_A  u( 5) = theta_yA  u( 6) = theta_zA
	% u(8) = y_B  u(9) = z_B  u(11) = theta_yB  u(12) = theta_zB
	% z(1) = phi_A            z( 2) = phi_B

	% First, transform to local coordinates
	% u = nldata.T * u;
	% v = nldata.T * v;

	% Put into "2D" variables to make it easier
	theta_A = - u( 5);   theta_A_dot = - v( 5);
	theta_B = - u(11);   theta_B_dot = - v(11);

	Qa = 2*z(1) +   z(2);
	Qb =   z(1) + 2*z(2);

	% I have tried to code the following for computational efficiency, not
	% necessarily coding efficiency or elegance.  Suggestions/fixes welcome.
	if abs(Qa) < Y % {
		if abs(Qb) < Y
			% no yielding (elastic-elastic)
			state=1;

		elseif Qb * ( theta_A_dot + 2 * theta_B_dot) < 0

			% Side B unloading: elastic-elastic
			state=1;
		else
			% A is not yielding, B must be yielding: elastic-plastic
			state=2;
		end
	% }

	elseif Qa * ( 2 * theta_A_dot + theta_B_dot) < 0 % {

		if abs(Qb) < Y
			% Side A unloading, side B not yielding: elastic-elastic
			state=1;

		elseif Qb * ( theta_A_dot + 2 * theta_B_dot - 3) < 0

			% Both unloading:  elastic-elastic
			state=1;

		elseif Qb * theta_B_dot < 0

			% Side B not yielding--angle across hinge is not right: plastic-elastic
			state=3;
		else
			% if we get here, it is plastic-plastic
			state=4;
		end
	% }
	elseif abs(Qb) < Y
		% B not yielding, A cannot be elastic: plastic-elastic
		state=3; 

	elseif Qa * theta_A_dot < 0 & Qb * (theta_A_dot + 2 * theta_B_dot) < 0

		% Side A not yielding--angle not right: elastic-plastic
		state=2;
	else
		% Out of options here: plastic-plastic
		state=4;
	end

function nl_data = nonlinear_data( element, material, section, nodelist)

	L  = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));
	E  = material.E;
	Iy = section.Iy;
	Iz = section.Iz;

	nl_data.L       = L;

	yvec = [3,5,9,11];   % degrees of freedom which depend on Iy (Fz,My)
	                     % which is the yielding axis

	nl_data.Kee = zeros(12);
	nl_data.Kee(yvec,yvec) = beamutil('lineary', E, Iy, L);

	nl_data.Kep = zeros(12);
	% bending about local z-axis assumed to stay linear
	nl_data.Kep(yvec,yvec) = beamutil('elastic-plasticy', E, Iy, L);

	nl_data.Kpe = zeros(12);
	nl_data.Kpe(yvec,yvec) = beamutil('plastic-elasticy', E, Iy, L);

	% "sparsify" the matrices
	nl_data.Kee = sparse( nl_data.Kee);
	nl_data.Kep = sparse( nl_data.Kep);
	nl_data.Kpe = sparse( nl_data.Kpe);

	% The yield moment
	nl_data.My      = material.Fy * section.Z;
	% moment capacity reached when  | 2 phiA + phiB - 3 psi | > Y
	nl_data.Y       = material.Fy * section.Z * L / 2 / E / Iz;
	nl_data.T       = beam3d( 'transformation-matrix', element, nodelist);
	nl_data.Tinv    = sparse( inv(nl_data.T));

	if isfield( material, 'alpha') & ~ isempty(material.alpha)
		nl_data.Kee    = (1-material.alpha) * nl_data.Kee;
		nl_data.Kep    = (1-material.alpha) * nl_data.Kep;
		nl_data.Kpe    = (1-material.alpha) * nl_data.Kpe;
	end

	% Global stiffness matrices
	nl_data.KEE = nl_data.Tinv * nl_data.Kee * nl_data.T;
	nl_data.KEP = nl_data.Tinv * nl_data.Kep * nl_data.T;
	nl_data.KPE = nl_data.Tinv * nl_data.Kpe * nl_data.T;

	% Create the mapping vectors and transform from the "inside rotation" to
	% the global coordinates.

% compute the local element forces
% D are the *global* displacements
function f = local_forces( element, material, section, nodelist, D, z)

	% Inside the hinges is linear (pre-yield)
	Klocal = pre_yield_stiffness_matrix(element, material, section, nodelist);
	T = beam3d( 'transformation-matrix', element, nodelist);

	% Need to use the state variables as the end rotations.
	% Negative because the y-axis points in the opposite direction of
	% the "2D" z-axis.
	if min( size(D)) > 1   % vectorized
		D( 5, :) = - z(1, :);
		D(11, :) = - z(2, :);
	else
		D( 5) = - z(1);
		D(11) = - z(2);
	end

	f = Klocal * T * D;
	f = matrix_clean(f);

% Note, assuming that yielding occurs at the beam end about the y-axis only
% TODO:  Verify this before using!! I have not tested it and I think it is broke.
function [K, zdot,state] = nonlinear_stiffness( element, material, section, nodelist, u, v, z, nldata)

	% Transform to local
	u = nl_data.T * u;
	v = nl_data.T * v;

	state = local_get_state( u, v, z, nldata.Y, nldata.L);

	% FIXME: Should these be global coordinates?  (I put the transformation in
	% above because local_get_state needs them that way.  Previously, the code
	% was using the global coordinates.

	% Put into "2D" variables to make it easier
	theta_A_dot = - v( 5);
	theta_B_dot = v(11);

	switch state
		case 1
			K = nldata.Kee;
			zdot = [ theta_A_dot; theta_B_dot ];   % zdot = theta_dot
		case 2
			K = nldata.Kep;
			zdot(1) = theta_A_dot;
			zdot(2) = ( - theta_A_dot + 3 * psi_dot) / 2;
		case 3
			K = nldata.Kpe;
			zdot(1) = ( - theta_B_dot + 3 * psi_dot) / 2;
			zdot(2) = theta_B_dot;
		case 4
			K = zeros(12);

			zdot = [0; 0];

		otherwise
			error('Invalid state returned from local_get_state');
	end

% {TEX} This is a two-node, three-dimensional, bilinear, ``simplified'' element.
% {TEX} Axial and torsional stiffnesses are neglected, thus a constraint or
% {TEX} a truss element must be used to provide axial stiffness if desired.
% {TEX} If the pre-yielding stiffness is given by $EI$, then
% {TEX} the post-yielding stiffness is given by $\alpha EI$.
% {TEX} A perfectly elastic-plastic element can be defined by setting $\alpha = 0$.
% {TEX} The state variables for this element are the angles of the ``elastic''
% {TEX} portions of the beam.
% {TEX}
% {TEX} Element axes are defined as outlined in Balfour (1992).
% {TEX} The element $x$ axis lies on the line joining the element nodes.
% {TEX} The $y$ axis is then oriented to lie in the global $x$,$y$ plane.
% {TEX} The $z$ axis is then defined using the ``right hand rule.''

% Required Material Fields
% {TEXRM} $E$, $alpha$, $Fy$
% Required Section Fields
% {TEXRS} $Iy$, $Iz$, $Z$

% Licence Details

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
