function varargout = vdamper2d( flag, varargin)    % TEXFEM

% SYNOPSIS:
%
% function varargout = vdamper3d(varargin)
%
% Three-dimensional, linear viscous element for use with the FE toolkit.
%
% See element_definition for details

% Copyright (C) 2001 Matthew W. Roberts
% Released under the GNU Public License (See end of file for details.)

% {TEX} This is a two-node, two-dimensional, linear viscous damping element.

switch flag
	case 'stiffness-matrix'
		varargout{1} = zeros(2);
	case 'damping-matrix'
		varargout{1} = local_damping(varargin{:});
	case 'transformation-matrix'
		varargout{1} = local_transform(varargin{:});
	case 'num-dof'
		varargout{1} = 2;   % two degrees of freedom per node
	case 'num-nodes'
		varargout{1} = 2;   % two nodes per element
	case 'dof-vector'  % equivalent to the dofs array of a FElt definition
		varargout{1} = [1 2];
	case 'element-forces'
		varargout{1} = local_forces( varargin{:});
	case 'element-forces-fast'   % Stiffness matrix passed in instead of assembled
		varargout{1} = local_forces_fast( varargin{:});
	case 'element-type'
		type.damping_element = 1;
		varargout{1} = type;
	otherwise
		warning(['Beam3d: Unknown option ''' flag '''.']);
end

% --------------------------------------------------------------------------

function C = local_damping(element, material, section, nodelist)

	% Required Material Fields
	% {TEXRM} $c$

	C = [ material.c  -material.c ; ...
	     -material.c   material.c];

function T = local_transform(element, nodelist)

T = zeros (2,4);

L = norm( nodelist( element.nodes(2),:) - nodelist( element.nodes(1),:));
if  L < eps
	error ('length of element is zero to machine precision');
end

dx = (nodelist(element.nodes(2),1) - nodelist(element.nodes(1),1))/L;
dy = (nodelist(element.nodes(2),2) - nodelist(element.nodes(1),2))/L;

T(1, 1) = dx;
T(1, 2) = dy;
T(2, 3) = dx;
T(2, 4) = dy;


% compute the element forces
% D are the *global* displacements
function f = local_forces( element, material, section, nodelist, D)

Klocal = local_damping(element, material, section, nodelist);
T = truss3d( 'transformation-matrix', element, nodelist);

f = Klocal * T * D;
f = matrix_clean(f);

function f = local_forces_fast( element, material, section, nodelist, D, KT)

f = KT * D;

% Licence Details and Credits:

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

% Adapted from the FElt Finite Element package by Jason I. Gobat
% and Darren C. Atkinson 
